package be.iminds.ilabt.jfed.rspec.model.imutable_impl;

import be.iminds.ilabt.jfed.rspec.model.LinkSetting;

import java.util.Objects;

public class ImmutableLinkSetting implements LinkSetting {
    private final ImmutableRspecInterface from;
    private final ImmutableRspecInterface to;
    private final Long capacity_kbps;
    private final Integer latency_ms;
    private final Double packetLoss;
    private final boolean shownIfNotActive;

    public ImmutableLinkSetting(ImmutableRspecInterface fromIface, ImmutableRspecInterface toIface, LinkSetting o) {
        this.from = fromIface;
        this.to = toIface;

        assert from != null;
        assert to != null;

        assert Objects.equals(o.getFromIface().getClientId(), getFromIface().getClientId());
        assert Objects.equals(o.getToIface().getClientId(), getToIface().getClientId());

        if (o.isCapacitySet())
            capacity_kbps = o.getCapacity_Kbps();
        else
            capacity_kbps = null;

        if (o.isLatencySet())
            latency_ms = o.getLatency_ms();
        else
            latency_ms = null;

        if (o.isPacketLossSet())
            packetLoss = o.getPacketLoss();
        else
            packetLoss = null;

        this.shownIfNotActive = o.isShownIfNotActive();
    }

    @Override
    public ImmutableRspecInterface getFromIface() {
        return from;
    }

    @Override
    public ImmutableRspecInterface getToIface() {
        return to;
    }

    @Override
    public long getCapacity_Kbps() {
        return capacity_kbps == null ? -1 : capacity_kbps;
    }

    @Override
    public void setCapacity_Kbps(Long value) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean isCapacitySet() {
        return capacity_kbps != null && capacity_kbps >= 0;
    }

    @Override
    public int getLatency_ms() {
        return latency_ms == null ? -1 : latency_ms;
    }

    @Override
    public void setLatency_ms(Integer value) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean isLatencySet() {
        return latency_ms != null && latency_ms >= 0;
    }

    @Override
    public double getPacketLoss() {
        return packetLoss == null ? -1.0 : packetLoss;
    }

    @Override
    public void setPacketLoss(Double value) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean isPacketLossSet() {
        return packetLoss != null && packetLoss >= 0.0;
    }

    @Override
    public boolean isActive() {
        return isCapacitySet() || isPacketLossSet() || isLatencySet();
    }

    @Override
    public boolean isShownIfNotActive() {
        return shownIfNotActive;
    }

    @Override
    public void setShownIfNotActive(boolean shownIfNotActive) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void setSameAs(LinkSetting o) {
        throw new UnsupportedOperationException();
    }
}
